@@ -3,8 +3,8 @@ from typing import Any, List, Optional
 import nextcord
 from nextcord.ext import commands
 
-from .constants import EmojiType, PageFormatType, SendKwargsType
-from .menus import ButtonMenu, Menu, button
+from .constants import PageFormatType, SendKwargsType
+from .menus import Button, ButtonMenu, Menu
 from .page_source import PageSource
 from .utils import First, Last, _cast_emoji
 
@@ -18,11 +18,12 @@ class MenuPagesBase(Menu):
         The current page that we are in. Zero-indexed
         between [0, :attr:`PageSource.max_pages`).
     """
-    FIRST_PAGE = '\N{BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR}\ufe0f'
-    PREVIOUS_PAGE = '\N{BLACK LEFT-POINTING TRIANGLE}\ufe0f'
-    NEXT_PAGE = '\N{BLACK RIGHT-POINTING TRIANGLE}\ufe0f'
-    LAST_PAGE = '\N{BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR}\ufe0f'
-    STOP = '\N{BLACK SQUARE FOR STOP}\ufe0f'
+
+    FIRST_PAGE = "\N{BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR}\ufe0f"
+    PREVIOUS_PAGE = "\N{BLACK LEFT-POINTING TRIANGLE}\ufe0f"
+    NEXT_PAGE = "\N{BLACK RIGHT-POINTING TRIANGLE}\ufe0f"
+    LAST_PAGE = "\N{BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR}\ufe0f"
+    STOP = "\N{BLACK SQUARE FOR STOP}\ufe0f"
 
     def __init__(self, source: PageSource, **kwargs):
         self._source = source
@@ -54,7 +55,8 @@ class MenuPagesBase(Menu):
 
         if not isinstance(source, PageSource):
             raise TypeError(
-                'Expected {0!r} not {1.__class__!r}.'.format(PageSource, source))
+                "Expected {0!r} not {1.__class__!r}.".format(PageSource, source)
+            )
 
         self._source = source
         self.current_page = 0
@@ -65,25 +67,40 @@ class MenuPagesBase(Menu):
     def should_add_reactions(self) -> bool:
         return self.should_add_reactions_or_buttons()
 
+    def should_add_buttons(self) -> bool:
+        return self.should_add_reactions_or_buttons()
+
     def should_add_reactions_or_buttons(self) -> bool:
         return self._source.is_paginating()
 
     async def _get_kwargs_from_page(self, page: List[Any]) -> SendKwargsType:
-        value: PageFormatType = await nextcord.utils.maybe_coroutine(self._source.format_page, self, page)
+        """|coro|
+
+        Calls :meth:`PageSource.format_page` and returns a dict of send kwargs
+        """
+        value: PageFormatType = await nextcord.utils.maybe_coroutine(
+            self._source.format_page, self, page
+        )
         if isinstance(value, dict):
             return value
         elif isinstance(value, str):
-            return {'content': value, 'embed': None}
+            return {"content": value, "embed": None}
         elif isinstance(value, nextcord.Embed):
-            return {'embed': value, 'content': None}
+            return {"embed": value, "content": None}
 
     async def show_page(self, page_number: int):
+        """|coro|
+
+        Sets the current page to the specified page and shows it.
+        """
         page = await self._source.get_page(page_number)
         self.current_page = page_number
         kwargs = await self._get_kwargs_from_page(page)
         await self.message.edit(**kwargs)
 
-    async def send_initial_message(self, ctx: commands.Context, channel: nextcord.abc.Messageable) -> nextcord.Message:
+    async def send_initial_message(
+        self, ctx: commands.Context, channel: nextcord.abc.Messageable
+    ) -> nextcord.Message:
         """|coro|
 
         The default implementation of :meth:`Menu.send_initial_message`
@@ -93,13 +110,37 @@ class MenuPagesBase(Menu):
         """
         page = await self._source.get_page(0)
         kwargs = await self._get_kwargs_from_page(page)
-        if hasattr(self, '__discord_ui_view__'):
-            kwargs['view'] = self
+        # filter out kwargs that are "None"
+        kwargs = {k: v for k, v in kwargs.items() if v is not None}
+        # if there is an interaction, send an interaction response
+        if self.interaction is not None:
+            await self.interaction.response.send_message(
+                ephemeral=self.ephemeral, **kwargs
+            )
+            return await self.interaction.original_message()
+        # otherwise, send the message using the channel
         return await channel.send(**kwargs)
 
-    async def start(self, ctx: commands.Context, *, channel: Optional[nextcord.abc.Messageable] = None, wait: Optional[bool] = False):
+    async def start(
+        self,
+        ctx: Optional[commands.Context] = None,
+        interaction: Optional[nextcord.Interaction] = None,
+        *,
+        channel: Optional[nextcord.abc.Messageable] = None,
+        wait: bool = False,
+        ephemeral: bool = False,
+    ):
         await self._source._prepare_once()
-        await super().start(ctx, channel=channel, wait=wait)
+        await super().start(
+            ctx=ctx,
+            interaction=interaction,
+            channel=channel,
+            wait=wait,
+            ephemeral=ephemeral,
+        )
+        # If we're not paginating, we can remove the pagination buttons
+        if not self._source.is_paginating():
+            await self.clear()
 
     async def show_checked_page(self, page_number: int):
         max_pages = self._source.get_max_pages()
@@ -121,6 +162,27 @@ class MenuPagesBase(Menu):
             return True
         return max_pages <= 2
 
+    async def go_to_first_page(self, payload=None):
+        """go to the first page"""
+        await self.show_page(0)
+
+    async def go_to_previous_page(self, payload=None):
+        """go to the previous page"""
+        await self.show_checked_page(self.current_page - 1)
+
+    async def go_to_next_page(self, payload=None):
+        """go to the next page"""
+        await self.show_checked_page(self.current_page + 1)
+
+    async def go_to_last_page(self, payload=None):
+        """go to the last page"""
+        # The call here is safe because it's guarded by skip_if
+        await self.show_page(self._source.get_max_pages() - 1)
+
+    async def stop_pages(self, payload=None):
+        """stops the pagination session."""
+        self.stop()
+
 
 class MenuPages(MenuPagesBase):
     """A special type of Menu dedicated to pagination with reactions.
@@ -134,37 +196,45 @@ class MenuPages(MenuPagesBase):
 
     def __init__(self, source: PageSource, **kwargs):
         super().__init__(source, **kwargs)
-
-    @button(MenuPagesBase.FIRST_PAGE, position=First(0), skip_if=MenuPagesBase._skip_double_triangle_buttons)
-    async def go_to_first_page(self, payload):
-        """go to the first page"""
-        await self.show_page(0)
-
-    @button(MenuPagesBase.PREVIOUS_PAGE, position=First(1))
-    async def go_to_previous_page(self, payload):
-        """go to the previous page"""
-        await self.show_checked_page(self.current_page - 1)
-
-    @button(MenuPagesBase.NEXT_PAGE, position=Last(0))
-    async def go_to_next_page(self, payload):
-        """go to the next page"""
-        await self.show_checked_page(self.current_page + 1)
-
-    @button(MenuPagesBase.LAST_PAGE, position=Last(1), skip_if=MenuPagesBase._skip_double_triangle_buttons)
-    async def go_to_last_page(self, payload):
-        """go to the last page"""
-        # The call here is safe because it's guarded by skip_if
-        await self.show_page(self._source.get_max_pages() - 1)
-
-    @button(MenuPagesBase.STOP, position=Last(2))
-    async def stop_pages(self, payload):
-        """stops the pagination session."""
-        self.stop()
+        # skip adding buttons if inherit_buttons=False was passed to metaclass
+        if not self.__inherit_buttons__:
+            return
+        # add pagination reaction buttons
+        self.add_button(
+            Button(
+                self.FIRST_PAGE,
+                self.go_to_first_page,
+                position=First(0),
+                skip_if=self._skip_double_triangle_buttons,
+            ),
+        )
+        self.add_button(
+            Button(self.PREVIOUS_PAGE, self.go_to_previous_page, position=First(1))
+        )
+        self.add_button(
+            Button(self.NEXT_PAGE, self.go_to_next_page, position=Last(0)),
+        )
+        self.add_button(
+            Button(
+                self.LAST_PAGE,
+                self.go_to_last_page,
+                position=Last(1),
+                skip_if=self._skip_double_triangle_buttons,
+            ),
+        )
+        self.add_button(
+            Button(self.STOP, self.stop_pages, position=Last(2)),
+        )
 
 
-class MenuPaginationButton(nextcord.ui.Button['MenuPaginationButton']):
+class MenuPaginationButton(nextcord.ui.Button["MenuPaginationButton"]):
     """
-    A custom button for pagination that will be disabled when unavailable.
+    A custom button for pagination used by :class:`ButtonMenuPages`
+    that runs pagination methods in the :meth:`MenuPaginationButton.callback`
+    corresponding to the emoji.
+
+    This is a subclass of :class:`nextcord.ui.Button` and as
+    such, accepts all of its parameters.
     """
 
     def __init__(self, **kwargs):
@@ -183,24 +253,16 @@ class MenuPaginationButton(nextcord.ui.Button['MenuPaginationButton']):
         view: ButtonMenuPages = self.view
 
         # change the current page
-        if self._emoji.name == view.FIRST_PAGE:
-            await view.show_page(0)
-        elif self._emoji.name == view.PREVIOUS_PAGE:
-            await view.show_checked_page(view.current_page - 1)
-        elif self._emoji.name == view.NEXT_PAGE:
-            await view.show_checked_page(view.current_page + 1)
-        elif self._emoji.name == view.LAST_PAGE:
-            await view.show_page(view._source.get_max_pages() - 1)
-
-        # disable buttons that are unavailable
-        view._disable_unavailable_buttons()
-
-        # disable all buttons if stop is pressed
-        if self._emoji.name == view.STOP:
-            return view.stop()
-
-        # update the view
-        await interaction.response.edit_message(view=view)
+        if str(self._emoji) == view.FIRST_PAGE:
+            await view.go_to_first_page()
+        elif str(self._emoji) == view.PREVIOUS_PAGE:
+            await view.go_to_previous_page()
+        elif str(self._emoji) == view.NEXT_PAGE:
+            await view.go_to_next_page()
+        elif str(self._emoji) == view.LAST_PAGE:
+            await view.go_to_last_page()
+        elif str(self._emoji) == view.STOP:
+            await view.stop_pages()
 
 
 class ButtonMenuPages(MenuPagesBase, ButtonMenu):
@@ -218,29 +280,63 @@ class ButtonMenuPages(MenuPagesBase, ButtonMenu):
         between [0, :attr:`PageSource.max_pages`).
     """
 
-    def __init__(self, source: PageSource, style: nextcord.ButtonStyle = nextcord.ButtonStyle.secondary, **kwargs):
+    def __init__(
+        self,
+        source: PageSource,
+        style: nextcord.ButtonStyle = nextcord.ButtonStyle.secondary,
+        **kwargs,
+    ):
         self.__button_menu_pages__ = True
         # make button pagination disable buttons on stop by default unless it's overridden
-        kwargs["disable_buttons_after"] = kwargs.get("disable_buttons_after", True)
+        if "disable_buttons_after" not in kwargs:
+            kwargs["disable_buttons_after"] = True
         super().__init__(source, **kwargs)
-        # skip adding buttons if inherit_buttons=False was passed to metaclass
-        if not self.__inherit_buttons__:
+        # skip adding buttons if inherit_buttons=False was passed to metaclass or only one page
+        if not self.__inherit_buttons__ or not self.should_add_buttons():
             return
         # add buttons to the view
-        for emoji in (self.FIRST_PAGE, self.PREVIOUS_PAGE, self.NEXT_PAGE, self.LAST_PAGE, self.STOP):
-            if emoji in {self.FIRST_PAGE, self.LAST_PAGE} and self._skip_double_triangle_buttons():
+        pagination_emojis = (
+            self.FIRST_PAGE,
+            self.PREVIOUS_PAGE,
+            self.NEXT_PAGE,
+            self.LAST_PAGE,
+            self.STOP,
+        )
+        double_triangle_emojis = {self.FIRST_PAGE, self.LAST_PAGE}
+        for emoji in pagination_emojis:
+            if emoji in double_triangle_emojis and self._skip_double_triangle_buttons():
                 continue
             self.add_item(MenuPaginationButton(emoji=emoji, style=style))
+        # disable buttons that are not available
         self._disable_unavailable_buttons()
 
+    async def show_page(self, page_number: int):
+        """|coro|
+        Sets the current page to the specified page and shows it.
+        """
+        # disable buttons that are not available
+        self.current_page = page_number
+        self._disable_unavailable_buttons()
+        # show the page
+        await super().show_page(page_number)
+
+    async def _get_kwargs_from_page(self, page: List[Any]) -> SendKwargsType:
+        """|coro|
+        Calls :meth:`PageSource.format_page` and returns a dict of send kwargs
+        """
+        kwargs = await super()._get_kwargs_from_page(page)
+        # add view to kwargs if it's not already there
+        return {"view": self, **kwargs}
+
     def _disable_unavailable_buttons(self):
         """
         Disables buttons that are unavailable to be pressed.
         """
-        buttons: List[MenuPaginationButton] = self.children
+        children: List[MenuPaginationButton] = self.children
         max_pages = self._source.get_max_pages()
-        for button in buttons:
-            if button.emoji.name in (self.FIRST_PAGE, self.PREVIOUS_PAGE):
-                button.disabled = self.current_page == 0
-            elif max_pages and button.emoji.name in (self.LAST_PAGE, self.NEXT_PAGE):
-                button.disabled = self.current_page == max_pages - 1
+        for child in children:
+            if isinstance(child, nextcord.ui.Button):
+                if str(child.emoji) in (self.FIRST_PAGE, self.PREVIOUS_PAGE):
+                    child.disabled = self.current_page == 0
+                elif max_pages and str(child.emoji) in (self.LAST_PAGE, self.NEXT_PAGE):
+                    child.disabled = self.current_page == max_pages - 1
